"use strict";
// Copyright (c) Rodrigo Speller. All rights reserved.
// Licensed under the Apache License, Version 2.0.
// See License.txt in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
const abstractions_1 = require("./abstractions");
const Entry_1 = require("./Entry");
const ActivatorContainer_1 = require("./ActivatorContainer");
class ContainerBase extends abstractions_1.Container {
    constructor(entries = new Entry_1.EntryMap()) {
        super();
        this.containerSymbol = Symbol();
        let containerSymbol = this.containerSymbol;
        entries.forEach((tag, entry) => entry.hold(containerSymbol));
        this.entries = entries;
    }
    getEntry(arg1) {
        let entry;
        if (arg1 instanceof Function) {
            entry = Entry_1.Entry.get(arg1, this.containerSymbol);
            if (entry == null) {
                let tag = Symbol();
                entry = this.createEntry(tag, arg1, Entry_1.TransientEntry);
                this.entries.set(tag, entry);
            }
        }
        else {
            entry = this.entries.get(arg1);
        }
        return entry;
    }
    createEntry(tag, factory, lifetime) {
        let entry = new lifetime(tag, factory);
        entry.hold(this.containerSymbol);
        return entry;
    }
    resolve(arg1) {
        return new ActivatorContainer_1.default(this).resolve(arg1);
    }
}
exports.default = ContainerBase;
