export declare const resolver: unique symbol;
export declare type Tag = string | symbol;
export interface SimpleDependencyType<T> {
	new (container: Container): T;
}
export interface ComplexDependencyType<T> {
	new (...args: any): T;
	[resolver](container: Container): T;
}
export declare type DependencyType<T> = SimpleDependencyType<T> | ComplexDependencyType<T>;
export declare type DependencyResolver<T> = (container: Container) => T;
export declare abstract class Container {
	abstract createScope(): Container;
	abstract resolve<T>(type: DependencyType<T>): T;
	abstract resolve<T>(tag: Tag): T;
}
export declare class ContainerBuilder {
	build(): Container;
	register<T>(type: DependencyType<T>): ContainerBuilder;
	register<T>(tag: Tag, type: DependencyType<T>): ContainerBuilder;
	register<T>(tag: Tag, factory: DependencyResolver<T>): ContainerBuilder;
	registerSingleton<T>(type: DependencyType<T>): ContainerBuilder;
	registerSingleton<T>(tag: Tag, type: DependencyType<T>): ContainerBuilder;
	registerSingleton<T>(tag: Tag, factory: DependencyResolver<T>): ContainerBuilder;
	registerScoped<T>(type: DependencyType<T>): ContainerBuilder;
	registerScoped<T>(tag: Tag, type: DependencyType<T>): ContainerBuilder;
	registerScoped<T>(tag: Tag, factory: DependencyResolver<T>): ContainerBuilder;
}
export default ContainerBuilder;